#!/usr/bin/awk -f

BEGIN{
  print "# 5-band spectra from the ADR3.dat (Sloan DSS MOC catalogue) <- multiple entries for one asteroid are possible!";
  print "# wavelength (nm) & relative reflectance (wrt. solar analog) normalized to 1 at 550 nm & reflectance error";
  print "## (a comment) astdys-like designation & PC1 & PC1_err & PC2 & PC2_err";

# Sloan filter bands (they are 0.1-0.3 microns wide)
  A = 0.1;	# nm
  lambda_u = 3557.*A;
  lambda_g = 4825.*A;
  lambda_r = 6261.*A;
  lambda_i = 7672.*A;
  lambda_z = 9097.*A;
  
# the solar analog:
  u_g_S = 1.32;
  g_r_S = 0.45;
  r_i_S = 0.10;
  i_z_S = 0.04;

  g_S = 0;	# an arbitrary constant (I will normalize fluxes anyway)
  u_S = u_g_S + g_S;
  r_S = g_S - g_r_S;
  i_S = r_S - r_i_S;
  z_S = i_S - i_z_S;

# principal components coefficients:
  a11 =  0.396; a12 = 0.553; a13 = 0.567; a14 = 0.465;
  a21 = -0.819; a22 = 0.017; a23 = 0.09 ; a24 = 0.567;

  ERR = 0.1;	# PC1, PC2 error cutoff (for the selection of good spectra)

  j=0;	# good spectra counter
}
!/^#/ && ($34 == 1){	# drop commented spectra in *.sloan
#($34 == 1){	# dbg
  if ($35 != 0) {
    dsg = $35;
  } else {
    dsg = gensub("_", "", 1, $36);
  }

  u = $20;
  g = $22;
  r = $24;
  i = $26;
  z = $28;
  uerr = $21;
  gerr = $23;
  rerr = $25;
  ierr = $27;
  zerr = $29;

# subtract the solar analog
  u = u - u_S;
  g = g - g_S;
  r = r - r_S;
  i = i - i_S;
  z = z - z_S;

# convert to fluxes (reflectances)
  E_u = mag2flux(u);
  E_g = mag2flux(g);
  E_r = mag2flux(r);
  E_i = mag2flux(i);
  E_z = mag2flux(z);

# interpolate and normalize to 550 nm
  E_550 = E_g + (E_r-E_g) * (550.-lambda_g)/(lambda_r-lambda_g)
  tmp = E_550;
#  tmp = E_g;	# <- this was a simple normalization to the g-band

  E_u = E_u/tmp;
  E_g = E_g/tmp;
  E_r = E_r/tmp;
  E_i = E_i/tmp;
  E_z = E_z/tmp;

# flux errors with a correct error propagation
  E_uerr = magerr2fluxerr(u,uerr);
  E_gerr = magerr2fluxerr(g,gerr);
  E_rerr = magerr2fluxerr(r,rerr);
  E_ierr = magerr2fluxerr(i,ierr);
  E_zerr = magerr2fluxerr(z,zerr);

# principal components and their errors
  PC1 =  a11*(u-g) + a12*(g-r) + a13*(g-i) + a14*(g-z);
  PC2 =  a21*(u-g) + a22*(g-r) + a23*(g-i) + a24*(g-z);

  PC1err =  sqrt( (a11*uerr)**2 + (a11*gerr)**2 + (a12*gerr)**2 + (a12*rerr)**2 + (a13*gerr)**2 + (a13*ierr)**2 + (a14*gerr)**2 + (a14*zerr)**2 );
  PC2err =  sqrt( (a21*uerr)**2 + (a21*gerr)**2 + (a22*gerr)**2 + (a22*rerr)**2 + (a23*gerr)**2 + (a23*ierr)**2 + (a24*gerr)**2 + (a24*zerr)**2 );

  out = "# " dsg "	" PC1 "	" PC1err "	" PC2 "	" PC2err "\n" \
    lambda_u "	" E_u "	" E_uerr "\n" \
    lambda_g "	" E_g "	" E_gerr "\n" \
    lambda_r "	" E_r "	" E_rerr "\n" \
    lambda_i "	" E_i "	" E_ierr "\n" \
    lambda_z "	" E_z "	" E_zerr "\n";
  print out;
  print "set label \" " dsg "\" at " lambda_z "," E_z " font \"Helvetica,8\"" >"sloan_spectra.out.lab";

  if ((PC1err < ERR) && (PC2err < ERR)) {

    print out >"sloan_spectra.out_good";
    print "set label \" " dsg "\" at " lambda_z "," E_z " font \"Helvetica,8\"" >"sloan_spectra.out_good.lab";

# save fluxes for the mean spectrum calculation
    j++;
    E_u_[j] = E_u;
    E_g_[j] = E_g;
    E_r_[j] = E_r;
    E_i_[j] = E_i;
    E_z_[j] = E_z;
  }
}
END{

#  calculate a mean GOOD spectrum
  n=j;
  sum_u = 0.;
  sum_g = 0.;
  sum_r = 0.;
  sum_i = 0.;
  sum_z = 0.;
  for (j=1; j<=n; j++) {
    sum_u += E_u_[j];
    sum_g += E_g_[j];
    sum_r += E_r_[j];
    sum_i += E_i_[j];
    sum_z += E_z_[j];
  }
  E_u = sum_u/n;
  E_g = sum_g/n;
  E_r = sum_r/n;
  E_i = sum_i/n;
  E_z = sum_z/n;

  sum_u = 0.;
  sum_g = 0.;
  sum_r = 0.;
  sum_i = 0.;
  sum_z = 0.;
  for (j=1; j<=n; j++) {
    sum_u = sum_u + (E_u_[j] - E_u)**2;
    sum_g = sum_g + (E_g_[j] - E_g)**2;
    sum_r = sum_r + (E_r_[j] - E_r)**2;
    sum_i = sum_i + (E_i_[j] - E_i)**2;
    sum_z = sum_z + (E_z_[j] - E_z)**2;
  }
  E_uerr = sqrt(sum_u/(n-1));
  E_gerr = sqrt(sum_g/(n-1));
  E_rerr = sqrt(sum_r/(n-1));
  E_ierr = sqrt(sum_i/(n-1));
  E_zerr = sqrt(sum_z/(n-1));

  out = "# a mean spectrum and 1-sigma dispersion calculated from " n " (`good') measurements\n"\
    lambda_u "	" E_u "	" E_uerr "	" n "\n" \
    lambda_g "	" E_g "	" E_gerr "\n" \
    lambda_r "	" E_r "	" E_rerr "\n" \
    lambda_i "	" E_i "	" E_ierr "\n" \
    lambda_z "	" E_z "	" E_zerr "\n";
  
  print out >"sloan_spectra.out_mean";
}

# Pogson equation
func mag2flux(mag){
  return (10.**(-0.4*mag));
}

# $z = f(x,y,\dots)$
# $\Delta z^2 = \sum ({\partial f\over\partial x)^2 \Delta x^2 + \dots$
func magerr2fluxerr(mag,magerr){
  return (log(10.)*0.4 * 10.**(-0.4*mag) * magerr);
#  return (10.**(-0.4*(mag-magerr)) - 10.**(-0.4*mag));	# souhlasi s predchozim vztahem
}


